/*
 * C
 *
 * Copyright 2014-2016 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/******************************************************************************
Includes   <System Includes> , "Project Includes"
 ******************************************************************************/

#include "LLMJVM_impl.h"
#include <stdlib.h>
#include "tx_api.h"
#include "microej_core.h"
#include "microej.h"
#include "time_hardware_timer.h"

/******************************************************************************
Private global variables
 ******************************************************************************/
#define MICROEJ_OS_TICKS			10	// 1 os tick = 10 ms

// Timer for scheduling next alarm
static TX_TIMER LLMJVM_wake_up_timer;

// Application time offset
static int64_t LLMJVM_application_time_offset = 0;

// Absolute time in ticks at which timer will be launched
static int64_t LLMJVM_next_wake_up_time = INT64_MAX;

/******************************************************************************
Private functions
 ******************************************************************************/

/**
 * @brief this function is a callback used when software timer overflowed
 * @param p_args useless
 */
void wake_up_timer_schedule_request_callback(ULONG p_args)
{
	LLMJVM_schedule();
}

/******************************************************************************
LLMJVM_IMPL_* functions
 ******************************************************************************/

int32_t LLMJVM_IMPL_initialize()
{
	ssp_err_t err;

	time_hardware_timer_initialize();

	// Open timer to scheduler alarm for the VM
	err = tx_timer_create(
			&LLMJVM_wake_up_timer,
			"MICROJVM_TMR_WAKEUP",
			wake_up_timer_schedule_request_callback,
			0,	// useless
			1,
			0,	// one-shot timer
			TX_NO_ACTIVATE);

	if (err != SSP_SUCCESS)
	{
		// error during OS timer creation
		return LLMJVM_ERROR;
	}

	return LLMJVM_OK;
}

// Once the task is started, save a handle to it.
int32_t LLMJVM_IMPL_vmTaskStarted()
{
	return LLMJVM_OK;
}

int32_t LLMJVM_IMPL_scheduleRequest(int64_t absoluteTime)
{
	ssp_err_t err;
	int64_t relativeTime;
	int64_t relativeTick; // relative from current time to 'absoluteTime'

	// First check if absolute time is lower than current schedule time
	if(absoluteTime < LLMJVM_next_wake_up_time)
	{
		// Save new alarm absolute time
		LLMJVM_next_wake_up_time = absoluteTime;

		// Stop current timer (no delay)
		err = tx_timer_deactivate(&LLMJVM_wake_up_timer);

		if (TX_SUCCESS != err)
		{
			// error during timer stop
			while(1);
		}

		// Determine relative time/tick
		relativeTime = absoluteTime - LLMJVM_IMPL_getCurrentTime(MICROEJ_TRUE);
		relativeTick = relativeTime / MICROEJ_OS_TICKS;

		if(relativeTick > 0)
		{
			// Schedule the new alarm

			// update timer
			err = tx_timer_change(&LLMJVM_wake_up_timer, (ULONG)relativeTick, 0 /*one-shot timer*/);

			if (TX_SUCCESS != err)
			{
				// error during timer update
				return LLMJVM_ERROR;
			}

			// launch timer
			err = tx_timer_activate(&LLMJVM_wake_up_timer);

			if (TX_SUCCESS != err)
			{
				// error during timer activation
				return LLMJVM_ERROR;
			}

			return LLMJVM_OK;
		}
		else
		{
			// 'absoluteTime' has been reached already, notify the VM now
			return LLMJVM_schedule();
		}
	}
	// else : nothing to do. An sooner alarm is scheduled.

	return LLMJVM_OK;
}

// Suspends the VM task if no semaphore is available
int32_t LLMJVM_IMPL_idleVM()
{
	UINT err;

	err = tx_semaphore_get(&LLMJVM_java_thread_semaphore, TX_WAIT_FOREVER);

	if (TX_SUCCESS != err)
	{
		return LLMJVM_ERROR;
	}

	return LLMJVM_OK;
}

// Wakes up the VM task and reset next wake up time
int32_t LLMJVM_IMPL_wakeupVM()
{
	UINT err;

	LLMJVM_next_wake_up_time = INT64_MAX;

	err = tx_semaphore_put(&LLMJVM_java_thread_semaphore);

	if (TX_SUCCESS != err)
	{
		return LLMJVM_ERROR;
	}

	return LLMJVM_OK;
}

int32_t LLMJVM_IMPL_ackWakeup()
{
	return LLMJVM_OK;
}

int32_t LLMJVM_IMPL_getCurrentTaskID()
{
	return (int32_t)tx_thread_identify();
}

// Sets application time
void LLMJVM_IMPL_setApplicationTime(int64_t t)
{
	int64_t currentTime = LLMJVM_IMPL_getCurrentTime(MICROEJ_TRUE);
	LLMJVM_application_time_offset = t - currentTime;
}

// Gets the system or the application time in milliseconds
int64_t LLMJVM_IMPL_getCurrentTime(uint8_t system)
{
    int64_t systemTime = time_hardware_timer_getCurrentTime();
    return system ? systemTime : systemTime + LLMJVM_application_time_offset;
}

// Gets the current system time in nanoseconds
int64_t LLMJVM_IMPL_getTimeNanos()
{
	return time_hardware_timer_getTimeNanos();
}

int32_t LLMJVM_IMPL_shutdown(void)
{
	// nothing to do
	return LLMJVM_OK;
}

/* End of File */
