/***********************************************************************************************************************
 * Copyright [2015] Renesas Electronics Corporation and/or its licensors. All Rights Reserved.
 * 
 * This file is part of Renesas SynergyTM Software Package (SSP)
 *
 * The contents of this file (the "contents") are proprietary and confidential to Renesas Electronics Corporation
 * and/or its licensors ("Renesas") and subject to statutory and contractual protections.
 *
 * This file is subject to a Renesas SSP license agreement. Unless otherwise agreed in an SSP license agreement with
 * Renesas: 1) you may not use, copy, modify, distribute, display, or perform the contents; 2) you may not use any name
 * or mark of Renesas for advertising or publicity purposes or in connection with your use of the contents; 3) RENESAS
 * MAKES NO WARRANTY OR REPRESENTATIONS ABOUT THE SUITABILITY OF THE CONTENTS FOR ANY PURPOSE; THE CONTENTS ARE PROVIDED
 * "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, AND NON-INFRINGEMENT; AND 4) RENESAS SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, OR
 * CONSEQUENTIAL DAMAGES, INCLUDING DAMAGES RESULTING FROM LOSS OF USE, DATA, OR PROJECTS, WHETHER IN AN ACTION OF
 * CONTRACT OR TORT, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THE CONTENTS. Third-party contents
 * included in this file may be subject to different terms.
 **********************************************************************************************************************/
/***********************************************************************************************************************
* File Name    : system_S7G2.c
* Description  : Initializes C runtime environment, system clocks, pins, and interrupts. Calls warm start functions.
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes   <System Includes> , "Project Includes"
***********************************************************************************************************************/
#include "bsp_api.h"
#include "r_elc.h"

/* This include file is need only for the 'temporary' fix to insure that the Ioport reference counter is zeroed before it
 * gets referenced. Ioport init is currently called before the C Runtime initialization takes place.
 * It will be removed when a more complete solution for this problem is added.
 */
#include "..\..\..\src\driver\r_ioport\hw\hw_ioport_private.h"


#if defined(BSP_MCU_GROUP_S7G2)
/***********************************************************************************************************************
Macro definitions
***********************************************************************************************************************/

/***********************************************************************************************************************
Typedef definitions
***********************************************************************************************************************/

/***********************************************************************************************************************
Exported global variables (to be accessed by other files)
***********************************************************************************************************************/
uint32_t SystemCoreClock;  /*!< System Clock Frequency (Core Clock)*/

#if defined(__GNUC__)
/* Generated by linker. */
extern uint32_t __etext;
extern uint32_t __data_start__;
extern uint32_t __data_end__;
extern uint32_t __bss_start__;
extern uint32_t __bss_end__;
extern uint32_t __ebss_start__;
extern uint32_t __ebss_end__;
#elif defined(__ICCARM__)
#pragma section=".bss"
#pragma section=".data"
#pragma section=".data_init"
#endif

#if defined(__ICCARM__)
/* This pragma will suppress IAR warnings for mixing enumerated types in the bsp_pin_cfg.h file. This will be fixed
 * in the future but for now the warning is suppressed since it has been verified that this mixing is not an issue. */
#pragma diag_suppress=Pa089
#endif

/* BSP Pin Configuration. Allocate BSP pin configuration table in this module. */
#define BSP_PIN_ALLOCATE
#include "bsp_pin_cfg.h"

#if defined(__ICCARM__)
/* Restore IAR settings in regards to Pa089 (mixing enumerated types) back to command-line option setting. */
#pragma diag_default=Pa089
#endif

/* IOPORT implementation for this BSP. */
extern const ioport_api_t g_ioport_on_ioport;

/***********************************************************************************************************************
Private global variables and functions
***********************************************************************************************************************/
static void bsp_section_zero(uint8_t * pstart, uint32_t bytes);
static void bsp_section_copy(uint8_t * psource, uint8_t * pdest, uint32_t bytes);

/** Currently this structure is not being used. Eventually it will be tool generated. */
static const elc_cfg_t g_elc_cfg =
{
    .autostart = true,
    .link_count  = 0UL,
    .link_list = NULL
};

#if defined(__ICCARM__)
#pragma weak R_BSP_WarmStart
void R_BSP_WarmStart(bsp_warm_start_event_t event);
#elif defined(__GNUC__)
void R_BSP_WarmStart (bsp_warm_start_event_t event) __attribute__ ((weak));
#endif

#if defined(__ICCARM__)
#pragma weak __main
void __main(void);
#elif defined(__GNUC__)
void __main(void) __attribute__ ((weak));
#endif

/** BSP initialization function **/
void __rt_entry(void);


/***********************************************************************************************************************
* Function Name: SystemCoreClockUpdate
* Description  : Update SystemCoreClock variable based on current clock settings.
* Arguments    : none
* Return Value : none
***********************************************************************************************************************/
void SystemCoreClockUpdate (void)
{
    SystemCoreClock = bsp_cpu_clock_get();
}

/***********************************************************************************************************************
* Function Name: SystemInit
* Description  : Setup MCU.
* Arguments    : none
* Return Value : none
***********************************************************************************************************************/
void SystemInit (void)
{
#if ( defined ( __GNUC__ ) && defined (__VFP_FP__) && !defined (__SOFTFP__) ) || \
    ( defined ( __ICCARM__ ) && defined ( __ARMVFP__ ) && (__FPU_PRESENT == 1) )

    /* Enable the Cortex-M4 FPU only when -mfloat-abi=hard.
       Code taken from Section 7.1, Cortex-M4 TRM (DDI0439C) */

    /* Set bits 20-23 to enable CP10 and CP11 coprocessor */
    SCB->CPACR |= (0xF << 20);

#endif

    /* Call Pre C runtime initialization hook. */
    R_BSP_WarmStart(BSP_WARM_START_PRE_C);

    /* Initialize grouped interrupts. */
    bsp_group_interrupt_open();

    /* Configure system clocks using CGC module. */
    bsp_clock_init();

    /* Initialize register protection. */
    bsp_register_protect_open();

    /* Temporary fix to initialize ioport reference counter to 0, needed before C runtime init. This will be removed
     * in the next release in favor of a more complete solution. */
    HW_IOPORT_Init_Reference_Counter();

    /* Initialize pins. */
    g_ioport_on_ioport.init(&g_bsp_pin_cfg);

    /* BSP initialization */
    __main();
}

/***********************************************************************************************************************
* Function Name: bsp_section_zero
* Description  : Zero out input section
* Arguments    : pstart -
*                    Start address of the section
*                bytes -
*                    Size of section in bytes
* Return Value : none
***********************************************************************************************************************/
static void bsp_section_zero (uint8_t * pstart, uint32_t bytes)
{
    while (bytes > (uint32_t)0)
    {
        bytes--;
        pstart[bytes] = (uint32_t)0;
    }
}

/***********************************************************************************************************************
* Function Name: bsp_section_copy
* Description  : Zero out input section
* Arguments    : psource -
*                    Address of where to copy data from
*                pdest -
*                    Address of where to copy data to
*                bytes -
*                    Size of section in bytes
* Return Value : none
***********************************************************************************************************************/
static void bsp_section_copy (uint8_t * psource, uint8_t * pdest, uint32_t bytes)
{
    uint32_t index;
    for (index = 0U; index < bytes; index++,pdest++,psource++)
    {
        *pdest = *psource;
    }
}

/***********************************************************************************************************************
* Function Name: R_BSP_WarmStart
* Description  : This function is called at various points during the startup process. This function is declared as a
*                weak symbol higher up in this file because it is meant to be overridden by a user implemented version.
*                One of the main uses for this function is to call functional safety code during the startup process.
*                To use this function just copy this function into your own code and modify it to meet your needs.
* Arguments    : event -
*                    Where at in the start up process the code is currently at
* Return Value : none
***********************************************************************************************************************/
void R_BSP_WarmStart (bsp_warm_start_event_t event)
{
    if (BSP_WARM_START_PRE_C == event)
    {
        /* C runtime environment has not been setup so you cannot use globals. System clocks and pins are not setup. */
    }
    else if (BSP_WARM_START_POST_C == event)
    {
        /* C runtime environment, system clocks, and pins are all setup. */
    }
    else
    {

    }
}

/***********************************************************************************************************************
* Function Name: __main
* Description  : This function is called after system clock initialization. It initialize bss and data sections
*                and call bsp initialization.
* Arguments    : none
* Return Value : none
***********************************************************************************************************************/
void __main()
{
    /* Initialize C runtime environment. */
    /* Zero out BSS */
#if defined(__GNUC__)
    bsp_section_zero((uint8_t *)&__bss_start__, ((uint32_t)&__bss_end__ - (uint32_t)&__bss_start__));
#elif defined(__ICCARM__)
    bsp_section_zero((uint8_t *)__section_begin(".bss"), (uint32_t)__section_size(".bss"));
#endif

    /* Copy initialized RAM data from ROM to RAM. */
#if defined(__GNUC__)
    bsp_section_copy((uint8_t *)&__etext,
                     (uint8_t *)&__data_start__,
                     ((uint32_t)&__data_end__ - (uint32_t)&__data_start__));
#elif defined(__ICCARM__)
    bsp_section_copy((uint8_t *)__section_begin(".data_init"),
                     (uint8_t *)__section_begin(".data"),
                     (uint32_t)__section_size(".data"));
    /* Copy functions to be executed from RAM. */
    #pragma section=".code_in_ram"
    #pragma section=".code_in_ram_init"
    bsp_section_copy((uint8_t *)__section_begin(".code_in_ram_init"),
                     (uint8_t *)__section_begin(".code_in_ram"),
                     (uint32_t)__section_size(".code_in_ram"));
#endif

    __rt_entry();
}

/***********************************************************************************************************************
* Function Name: __rt_entry
* Description  : This function launch bsp initialization.
* Arguments    : none
* Return Value : none
***********************************************************************************************************************/
void __rt_entry()
{
    /* Initialize SystemCoreClock variable. */
    SystemCoreClock = bsp_cpu_clock_get();

    /* Call Post C runtime initialization hook. */
    R_BSP_WarmStart(BSP_WARM_START_POST_C);

    /* Initialize the Hardware locks to 'Unlocked' */
    bsp_init_hardware_locks();

    /* Initialize ELC events that will be used to trigger NVIC interrupts. */
    bsp_irq_cfg();

    /* Initialize ELC. */
    g_elc_on_elc.init(&g_elc_cfg);

    /* Call any BSP specific code. No arguments are needed so NULL is sent. */
    bsp_init(NULL);

    /* Zero out BSS placed in external SDRAM */
#if defined(__GNUC__)
    bsp_section_zero((uint8_t *)&__ebss_start__, ((uint32_t)&__ebss_end__ - (uint32_t)&__ebss_start__));
#elif defined(__ICCARM__)
    bsp_section_zero((uint8_t *)__section_begin(".ebss"), (uint32_t)__section_size(".ebss"));
#endif
}

void platform_main(void)
{
    __rt_entry();
}

#endif
