/* 
 * C
 * 
 * Copyright 2015-2016 IS2T. All rights reserved.
 * Modification and distribution is permitted under certain conditions.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

#include <nx_bsd.h>
#include "LLDISPATCH_CONSTANTS.h"
#include "LLDISPATCH_impl.h"
#include "LLDISPATCH_TIME_impl.h"
#include <stdlib.h>
#include <string.h>
#include "tx_api.h"
#include "LLNET_CONSTANTS.h"

static uint8_t NET_EMBEDDED_MUTEX_NAME[] = "NetEmbedded mutex";

static int8_t LLDISPATCH_mutex_initialized = 0;
static TX_MUTEX LLDISPATCH_mutex;
TX_SEMAPHORE   LLDISPATCH_semaphore;

static uint8_t isSleeping = 0;

static fd_set read_fds;
static fd_set write_fds;
static fd_set err_fds;


/***********************************************************************
 * External functions
 ***********************************************************************/

// function used to retrieve current time
extern int64_t LLMJVM_IMPL_getCurrentTime__Z(uint8_t);

extern TX_THREAD net_thread;

/***********************************************************************
 * External variables
 ***********************************************************************/

extern TX_EVENT_FLAGS_GROUP    nx_bsd_events;

void (*dispatchRunMethod)(void*) = NULL;


/***********************************************************************
 * Public functions
 ***********************************************************************/

void LLDISPATCH_IMPL_lock(void)
{
    int32_t status;
	if(!LLDISPATCH_mutex_initialized)
	{
		status = tx_mutex_create(&LLDISPATCH_mutex, NET_EMBEDDED_MUTEX_NAME, TX_NO_INHERIT);
		LLDISPATCH_mutex_initialized = 1;
		if(status != TX_SUCCESS){
			LLNET_DEBUG_ASSERT("[ERROR] Lock creation failed...\n");
		}
	}

	status = tx_mutex_get(&LLDISPATCH_mutex, TX_WAIT_FOREVER);
	if(status != TX_SUCCESS){
		LLNET_DEBUG_ASSERT("[ERROR] Lock failed...\n");
	}
}

void LLDISPATCH_IMPL_unlock(void)
{
    int32_t status = tx_mutex_put(&LLDISPATCH_mutex);
	if(status != TX_SUCCESS){
		// an error occurred
		LLNET_DEBUG_ASSERT("[ERROR] Unlock failed...\n");
	}
}

void LLDISPATCH_IMPL_startThread(void* method)
{
    tx_semaphore_create(&LLDISPATCH_semaphore, "LLDISPATCH semaphore", 0);

    //initialize dispatch run method ptr
	dispatchRunMethod = method;
	tx_thread_resume(&net_thread);
}

void LLDISPATCH_IMPL_wakeUpWaitForOperation(void)
{

	LLNET_DEBUG_TRACE("%s\n", __func__);
	int32_t status;
	if(isSleeping){
	    isSleeping = 0;
		status = tx_semaphore_put(&LLDISPATCH_semaphore);
	}else{
		/* Wakeup the thread that are attempting to perform a receive or that had their select satisfied.  */
		//sets NX_BSD_SELECT_EVENT flag to unblock the select operation
		status = tx_event_flags_set(&nx_bsd_events, NX_BSD_SELECT_EVENT, TX_OR);
	}
	if(status != TX_SUCCESS){
		LLNET_DEBUG_ASSERT("LLDISPATCH_IMPL_wakeUpWaitForOperation Error on unblocking select!!!!");
	}
}

int32_t LLDISPATCH_IMPL_waitForOperation(int32_t* fds, int32_t* fdOperations, int32_t fdsPtr)
{
	LLNET_DEBUG_TRACE("%s\n", __func__);
	int32_t max_fd = 0;
	if(fdsPtr == -1){
		// no file descriptor is waiting for operation, put the task in sleeping mode
		isSleeping = 1;
		tx_semaphore_get(&LLDISPATCH_semaphore, TX_WAIT_FOREVER);
	}else{
		LLDISPATCH_IMPL_lock();
		{
			FD_ZERO(&read_fds);
			FD_ZERO(&write_fds);
			FD_ZERO(&err_fds);
			int32_t fdIndex;


			// add read/write waiting operations in file descriptors select list
			for(fdIndex = fdsPtr; fdIndex>=0; --fdIndex){
				int32_t fd = fds[fdIndex];
				if(fdOperations[fdIndex] == LLDISPATCH_CONSTANTS_REQUEST_READ){
					FD_SET(fd, &read_fds);
				}else if(fdOperations[fdIndex] == LLDISPATCH_CONSTANTS_REQUEST_WRITE){
					FD_SET(fd, &write_fds);
				}else{
					LLNET_DEBUG_ASSERT("LLDISPATCH_IMPL_waitForOperation Wrong fd operation ERROR!!!!\n");
				}

				if(fd > max_fd){
					max_fd = fd;
				}
			}
		}
		LLDISPATCH_IMPL_unlock();

		LLNET_DEBUG_TRACE("- select\n");
		int32_t res = select(max_fd + 1, &read_fds, &write_fds, &err_fds, NULL);

		//checks if the select result
		if(res >= 0 ){
		    LLNET_DEBUG_TRACE("LLDISPATCH_CONSTANTS_WAIT_FOR_OPERATION_SUCCEED\n");
			return LLDISPATCH_CONSTANTS_WAIT_FOR_OPERATION_SUCCEED;
		}

		LLNET_DEBUG_ASSERT("One of the set contain an invalid file descriptor (err: %d)\n", res);
	}

	LLNET_DEBUG_TRACE("LLDISPATCH_CONSTANTS_WAIT_FOR_OPERATION_INTERRUPTED\n");
	return LLDISPATCH_CONSTANTS_WAIT_FOR_OPERATION_INTERRUPTED;
}

uint8_t LLDISPATCH_IMPL_isReadyForOperation(int32_t fd, int32_t operation)
{
	LLNET_DEBUG_TRACE("%s\n", __func__);
	if((operation == LLDISPATCH_CONSTANTS_REQUEST_READ) && (FD_ISSET(fd, &read_fds))){
		return 1;
	}else if((operation == LLDISPATCH_CONSTANTS_REQUEST_WRITE) && (FD_ISSET(fd, &write_fds))){
		return 1;
	}
	// else file descriptor not ready for operation
	return 0;
}

int64_t LLDISPATCH_TIME_IMPL_getCurrentTime(void)
{
	return LLMJVM_IMPL_getCurrentTime__Z(1); // 1 means that system time is required
}

