/*
 * C
 *
 * Copyright 2015-2016 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/* Includes ------------------------------------------------------------------*/

#include <stdio.h>
#include <stdint.h>
#include "LLDISPLAY_configuration.h"

#include "LLDISPLAY_IMPL.h"
#include "LLDISPLAY_EXTRA_IMPL.h"
#include "LLDISPLAY_EXTRA_drawing.h"
#include "microej.h"
#include "r_glcd.h"
#include "r_ioport.h"
#include "tx_api.h"
#include "bsp_util.h"
#include "microej_core.h"

/* Macros and Defines --------------------------------------------------------*/

/*
 * Size of physical display
 */
#define LLDISPLAY_PHYSICAL_WIDTH	(480)
#define LLDISPLAY_PHYSICAL_HEIGHT	(272)

/*
 * Default size of virtual display
 */
#ifndef LLDISPLAY_LOGICAL_WIDTH
#define LLDISPLAY_LOGICAL_WIDTH LLDISPLAY_PHYSICAL_WIDTH
#endif
#ifndef LLDISPLAY_LOGICAL_HEIGHT
#define LLDISPLAY_LOGICAL_HEIGHT LLDISPLAY_PHYSICAL_HEIGHT
#endif

/*
 * Sanity checks
 */
#if LLDISPLAY_LOGICAL_WIDTH > LLDISPLAY_PHYSICAL_WIDTH
#error "Invalid LLDISPLAY_LOGICAL_WIDTH"
#endif
#if LLDISPLAY_LOGICAL_HEIGHT > LLDISPLAY_PHYSICAL_HEIGHT
#error "Invalid LLDISPLAY_LOGICAL_HEIGHT"
#endif

/*
 * Set "fullscreen" mode
 */
#if (LLDISPLAY_LOGICAL_WIDTH == LLDISPLAY_PHYSICAL_WIDTH) && (LLDISPLAY_LOGICAL_HEIGHT == LLDISPLAY_PHYSICAL_HEIGHT)
#define LLDISPLAY_FULLSCREEN
#endif

/*
 * Ensure to use DMA when D2 engine is disabled (do not change it)
 */
#ifndef USE_D2ENGINE
#define USE_DMA
#endif


#ifdef USE_D2ENGINE
#include "dave_driver.h"
#endif
/*
 * DMA data
 */
#ifdef USE_DMA
#include "r_dmac.h"
#define PIXEL_BY_DMA_TRANSFERT (32/LLDISPLAY_BPP)
#define DMA_MAX_LLDISPLAY_LOGICAL_HEIGHT_BY_TRANSFERT (0xffff * (PIXEL_BY_DMA_TRANSFERT) / LLDISPLAY_LOGICAL_WIDTH)
#endif

/*
 * Macros to fill the remaining area in window screen mode
 */
#ifndef LLDISPLAY_FULLSCREEN
#define LLDISPLAY_COLOR(color, shift)	(((color) & (1 << (shift)) == 0 ? (color) : (color) | ((1 << (shift)) - 1)) & 0xff)
#define LLDISPLAY_RED(color) (LLDISPLAY_COLOR((color) >> 16, 3))
#define LLDISPLAY_GREEN(color) (LLDISPLAY_COLOR((color) >> 8, 2))
#define LLDISPLAY_BLUE(color) (LLDISPLAY_COLOR((color), 3))
#endif

/*
 * Declare MicroEJ Display buffers
 */
BSP_DECLARE_BUFFER(DISPLAY_MEM)
#define BUFFER_SIZE (LLDISPLAY_LOGICAL_WIDTH*LLDISPLAY_LOGICAL_HEIGHT*LLDISPLAY_BPP/8)
#define BACK_BUFFER ((int32_t)DISPLAY_MEM_START)
#define FRAME_BUFFER ((int32_t)(BACK_BUFFER + BUFFER_SIZE))
#define WORKINGBUFFER_START ((int32_t)(FRAME_BUFFER + BUFFER_SIZE))
#define WORKINGBUFFER_END ((int32_t)(DISPLAY_MEM_END))

/*
 * Constants according BPP
 */
#if LLDISPLAY_BPP == 16
#define LLDISPLAY_DISPLAY_IN_FORMAT DISPLAY_IN_FORMAT_16BITS_RGB565
#ifdef USE_D2ENGINE
#define LLDISPLAY_d2_mode d2_mode_rgb565
#endif
#elif LLDISPLAY_BPP == 32
#define LLDISPLAY_DISPLAY_IN_FORMAT DISPLAY_IN_FORMAT_32BITS_ARGB8888
#ifdef USE_D2ENGINE
#define LLDISPLAY_d2_mode d2_mode_argb8888
#endif
#else
#error "Invalid BPP"
#endif

/* Type ----------------------------------------------------------------------*/

/*
 * Flush mode status
 */
typedef enum flush_mode
{
	FLUSH_MODE_SLEEP,
	FLUSH_MODE_READY,
	FLUSH_MODE_RUNNING
} flush_mode_t;

/* Global --------------------------------------------------------------------*/

/*
 * DMA driver global variables
 */
#ifdef USE_DMA
extern const transfer_cfg_t ui_dma_cfg;
extern transfer_ctrl_t ui_dma_ctrl;
static volatile int32_t dma_ymin;
static volatile int32_t dma_ymax;
static volatile int32_t dma_dest_addr;
#endif

#ifdef USE_D2ENGINE
static d2_device *d2_handle;
#endif

TX_SEMAPHORE sync_semaphore;
static volatile flush_mode_t flush_mode;
static display_runtime_cfg_t disp_rt_cfg;

/* Private API ---------------------------------------------------------------*/

#ifdef USE_DMA

static void dma_reinit(int32_t srcAddress, uint8_t start_immediately)
{
	int32_t height = (dma_ymax - dma_ymin + 1);

	if (height > 0)
	{
		int32_t offset =  dma_ymin * LLDISPLAY_LOGICAL_WIDTH * LLDISPLAY_BPP/8;
		srcAddress += offset;
        int32_t destAddress = dma_dest_addr + offset;

		if (height > DMA_MAX_LLDISPLAY_LOGICAL_HEIGHT_BY_TRANSFERT)
		{
			// crop to max value
			height = DMA_MAX_LLDISPLAY_LOGICAL_HEIGHT_BY_TRANSFERT;
			dma_ymin += DMA_MAX_LLDISPLAY_LOGICAL_HEIGHT_BY_TRANSFERT;
		}
		else
		{
			// no more transfer after new one
			dma_ymin = dma_ymax + 1;
		}

		ui_dma.p_api->stop(&ui_dma_ctrl);
		ui_dma.p_api->reset(&ui_dma_ctrl, (void*)srcAddress, (void*)destAddress, (uint16_t)(LLDISPLAY_LOGICAL_WIDTH*height/PIXEL_BY_DMA_TRANSFERT));

		if (start_immediately)
		{
		    ui_dma.p_api->start(&ui_dma_ctrl, TRANSFER_START_MODE_REPEAT);
		}
	}
	else
	{
		// transfer done
		flush_mode = FLUSH_MODE_SLEEP;
		tx_semaphore_put(&sync_semaphore);
	}
}

#endif

/* Interrupts callbacks ------------------------------------------------------*/

void lcdc_callback (display_callback_args_t * p_args)
{
    if (flush_mode == FLUSH_MODE_READY && DISPLAY_EVENT_LINE_DETECTION == p_args->event)
    {
        // a copy is required
#ifdef USE_DMA
        flush_mode = FLUSH_MODE_RUNNING;
        ui_dma.p_api->start(&ui_dma_ctrl, TRANSFER_START_MODE_REPEAT);
#else
        flush_mode = FLUSH_MODE_SLEEP;
        d2_startframe(d2_handle);
        UINT ret = tx_semaphore_put(&sync_semaphore);
        if (ret != TX_SUCCESS)
        {
            while(1);
        }
#endif
    }
    //else: do nothing
}

#ifdef USE_DMA
void dma_callback(transfer_callback_args_t *p_args)
{
    // start next copy (if required)
    dma_reinit(((int32_t)disp_rt_cfg.input.p_base), 1);
}
#endif

/* API -----------------------------------------------------------------------*/

void LLDISPLAY_IMPL_initialize(void)
{
	// semaphore create
	tx_semaphore_create(&sync_semaphore, NULL, 0);

	// reset flush mode
    flush_mode = FLUSH_MODE_SLEEP;

#ifdef USE_DMA
    // initialize DMA engine

    // change the default interrupt flag
    extern transfer_info_t ui_dma_info;
    ui_dma_info.irq = TRANSFER_IRQ_END;

    ui_dma.p_api->open(&ui_dma_ctrl, &ui_dma_cfg);
#endif

    // controls the GPIO pin for LCD ON
    g_ioport.p_api->pinWrite(IOPORT_PORT_07_PIN_10, IOPORT_LEVEL_HIGH);

    // opens PWM driver and controls the TFT panel back light
    ui_backlight.p_api->open(ui_backlight.p_ctrl, ui_backlight.p_cfg);

    /*
     * We don't want to use the generated frame buffers allocation. So we disable the use
     * of graphics buffers (see Synergy configuration of "Display Driver on r_glcd") and
     * we set our own frame buffer address. Nevertheless, the following line throws a
     * compilation error:
     *
     * ui_lcdc.p_cfg->input[DISPLAY_FRAME_LAYER_1].p_base = (uint32_t*)FRAME_BUFFER;
     *
     * because the structure "ui_lcdc" is a "const" and the field "p_cfg" too. So we
     * create a local copy of this structure, set our frame buffer address and open the
     * graphics driver with this local configuration.
     */
    extern const display_cfg_t ui_lcdc_cfg;   // original configuration
    display_cfg_t ui_lcdc_cfg_copy;           // copy of configuration
    memcpy((void*)&ui_lcdc_cfg_copy, (void*)&ui_lcdc_cfg, sizeof(display_cfg_t));
    ui_lcdc_cfg_copy.input[DISPLAY_FRAME_LAYER_1].p_base = (uint32_t*)FRAME_BUFFER;   // set our address

    /*
     * Change too the size of frame area according LLDISPLAY_LOGICAL_*
     */
    ui_lcdc_cfg_copy.input[DISPLAY_FRAME_LAYER_1].hsize = LLDISPLAY_LOGICAL_WIDTH;
    ui_lcdc_cfg_copy.input[DISPLAY_FRAME_LAYER_1].hstride = LLDISPLAY_LOGICAL_WIDTH;
    ui_lcdc_cfg_copy.input[DISPLAY_FRAME_LAYER_1].vsize = LLDISPLAY_LOGICAL_HEIGHT;
    ui_lcdc_cfg_copy.input[DISPLAY_FRAME_LAYER_1].format = LLDISPLAY_DISPLAY_IN_FORMAT;
    ui_lcdc_cfg_copy.layer[DISPLAY_FRAME_LAYER_1].coordinate.x = (LLDISPLAY_PHYSICAL_WIDTH - LLDISPLAY_LOGICAL_WIDTH) / 2;
    ui_lcdc_cfg_copy.layer[DISPLAY_FRAME_LAYER_1].coordinate.y = (LLDISPLAY_PHYSICAL_HEIGHT - LLDISPLAY_LOGICAL_HEIGHT) / 2;

    // open and start the graphics driver
    ui_lcdc.p_api->open(ui_lcdc.p_ctrl, &ui_lcdc_cfg_copy);
    ui_lcdc.p_api->start(ui_lcdc.p_ctrl);

    // prepare frame buffers swapping
    disp_rt_cfg.input = ui_lcdc_cfg_copy.input[DISPLAY_FRAME_LAYER_1];
    disp_rt_cfg.layer = ui_lcdc_cfg_copy.layer[DISPLAY_FRAME_LAYER_1];

#ifdef USE_D2ENGINE
    // open D2 engine
    d2_handle = d2_opendevice(0);
    d2_inithw(d2_handle, 0);
    d2_startframe (d2_handle);  // Start execution of current render buffer commands
    d2_flushframe (d2_handle);  // Wait for rendering to complete
#endif
}

int32_t LLDISPLAY_IMPL_getWorkingBufferStartAddress(void)
{
	return WORKINGBUFFER_START;
}

int32_t LLDISPLAY_IMPL_getWorkingBufferEndAddress(void)
{

	return WORKINGBUFFER_END;
}

int32_t LLDISPLAY_IMPL_getGraphicsBufferAddress(void)
{
	return BACK_BUFFER;
}

int32_t LLDISPLAY_IMPL_getHeight(void)
{
	return LLDISPLAY_LOGICAL_HEIGHT;
}

int32_t LLDISPLAY_IMPL_getWidth(void)
{
	return LLDISPLAY_LOGICAL_WIDTH;
}

int32_t LLDISPLAY_IMPL_flush(int32_t addr, int32_t xmin, int32_t ymin, int32_t xmax, int32_t ymax)
{
	// prepare the copy from new display frame buffer into back buffer
	// (the copy will be launched in LCD interrupt callback)

#ifdef USE_DMA
	dma_ymin = ymin;
	dma_ymax = ymax;
	dma_dest_addr = ((int32_t)disp_rt_cfg.input.p_base);
	dma_reinit(addr, 0);
#else
    int32_t height = (ymax - ymin + 1);

    d2_setalpha(d2_handle, 0xff);   // reset alpha
    d2_framebuffer(d2_handle,disp_rt_cfg.input.p_base, LLDISPLAY_LOGICAL_WIDTH, LLDISPLAY_LOGICAL_WIDTH, LLDISPLAY_LOGICAL_HEIGHT, LLDISPLAY_d2_mode);
    d2_setblitsrc(d2_handle,(void*)addr, LLDISPLAY_LOGICAL_WIDTH, LLDISPLAY_LOGICAL_WIDTH, LLDISPLAY_LOGICAL_HEIGHT, LLDISPLAY_d2_mode);
    d2_blitcopy(d2_handle,
            LLDISPLAY_LOGICAL_WIDTH,
            height,
            0,ymin,
            D2_FIX4(LLDISPLAY_LOGICAL_WIDTH),
            D2_FIX4(height),
            D2_FIX4(0),D2_FIX4(ymin),
            d2_bf_no_blitctxbackup);
    d2_endframe(d2_handle);
#endif

#ifndef LLDISPLAY_FULLSCREEN
	// fix background color
	int32_t pix;
#if LLDISPLAY_BPP == 16
	pix = *((uint16_t*)addr);

	// task 15039: should use LLDISPLAY_UTILS function
	// for the moment: convert color on the fly
	pix = (int32_t)((pix & 0xf800) << 8 | (pix & 0x07e0) << 5 | (pix & 0x001f) << 3 | (int32_t)0xff000000);
#else
	pix = *((uint32_t*)addr);
#endif

    R_GLCDC->BACKGROUND.FILL_COLOR_b.BLUE  = LLDISPLAY_BLUE(pix);
    R_GLCDC->BACKGROUND.FILL_COLOR_b.GREEN = LLDISPLAY_GREEN(pix);
    R_GLCDC->BACKGROUND.FILL_COLOR_b.RED   = LLDISPLAY_RED(pix);
#endif

	// Buffer swapping
    int32_t ret = (int32_t)disp_rt_cfg.input.p_base;
	disp_rt_cfg.input.p_base= (uint32_t *)addr;
	ui_lcdc.p_api->layerChange(ui_lcdc.p_ctrl, &disp_rt_cfg, DISPLAY_FRAME_LAYER_1);

	// notify LCDC interrupt callback
	flush_mode = FLUSH_MODE_READY;

	return ret;
}

void LLDISPLAY_IMPL_synchronize(void)
{
	tx_semaphore_get(&sync_semaphore, TX_WAIT_FOREVER);
#ifndef USE_DMA
	d2_flushframe (d2_handle);  // Wait for rendering to complete
#endif
}


#ifdef USE_D2ENGINE
void LLDISPLAY_EXTRA_IMPL_waitPreviousDrawing(void)
{
    d2_flushframe (d2_handle);  // Wait for rendering to complete
}

int32_t LLDISPLAY_EXTRA_IMPL_fillRect(void* dest, int32_t destAddr, void* rect, int32_t color)
{
    destAddr += ((LLDISPLAY_SRectangle*)rect)->y * ((LLDISPLAY_SImage*)dest)->width * LLDISPLAY_BPP / 8;

    d2_framebuffer(d2_handle,(void*)destAddr, ((LLDISPLAY_SImage*)dest)->width, ((LLDISPLAY_SImage*)dest)->width, ((LLDISPLAY_SRectangle*)rect)->height, LLDISPLAY_d2_mode);
    d2_setcolor(d2_handle, 0, (d2_color)color);
    d2_renderbox(d2_handle, D2_FIX4(((LLDISPLAY_SRectangle*)rect)->x), D2_FIX4(0), D2_FIX4(((LLDISPLAY_SRectangle*)rect)->width), D2_FIX4(((LLDISPLAY_SRectangle*)rect)->height));
    d2_endframe(d2_handle);     // end of sequence
    d2_startframe(d2_handle);   // start new sequence to force to play previous sequence
    return LLDISPLAY_EXTRA_DRAWING_RUNNING;
}

int32_t LLDISPLAY_EXTRA_IMPL_drawImage(void* src, int32_t srcAddr, void* dest, int32_t destAddr, void* drawing)
{
    uint32_t format, blitflags, alpha;

    // for standard formats, alpha is a value between 0x00 and 0xff (only global alpha)
    alpha = ((LLDISPLAY_SDrawImage*)drawing)->alpha;

    format = ((LLDISPLAY_SImage*)src)->format;
    blitflags = d2_bf_no_blitctxbackup;

    switch(format)
    {
    case LLDISPLAY_EXTRA_IMAGE_RGB565:
        format = d2_mode_rgb565;
        break;
    case LLDISPLAY_EXTRA_IMAGE_ARGB8888:
        format = d2_mode_argb8888;
        blitflags |= d2_bf_usealpha;
        break;
    case LLDISPLAY_EXTRA_IMAGE_ARGB1555:
        format = d2_mode_argb1555;
        blitflags |= d2_bf_usealpha;
        break;
    case LLDISPLAY_EXTRA_IMAGE_ARGB4444:
        format = d2_mode_argb4444;
        blitflags |= d2_bf_usealpha;
        break;
    default:
        // should never occur (checked by caller)
        return LLDISPLAY_EXTRA_NOT_SUPPORTED;
    }

    d2_setalpha(d2_handle, (d2_alpha)alpha);
    d2_framebuffer(d2_handle,
            (void*)destAddr,
            ((LLDISPLAY_SImage*)dest)->width,
            ((LLDISPLAY_SImage*)dest)->width,
            ((LLDISPLAY_SImage*)dest)->height,
            LLDISPLAY_d2_mode);

    d2_setblitsrc(d2_handle,
            (void*)srcAddr,
            ((LLDISPLAY_SImage*)src)->width,
            ((LLDISPLAY_SImage*)src)->width,
            ((LLDISPLAY_SImage*)src)->height,
            format);

    d2_blitcopy(d2_handle,
            ((LLDISPLAY_SDrawImage*)drawing)->src_width,
            ((LLDISPLAY_SDrawImage*)drawing)->src_height,
            ((LLDISPLAY_SDrawImage*)drawing)->src_x,
            ((LLDISPLAY_SDrawImage*)drawing)->src_y,
            D2_FIX4(((LLDISPLAY_SDrawImage*)drawing)->src_width),
            D2_FIX4(((LLDISPLAY_SDrawImage*)drawing)->src_height),
            D2_FIX4(((LLDISPLAY_SDrawImage*)drawing)->dest_x),
            D2_FIX4(((LLDISPLAY_SDrawImage*)drawing)->dest_y),
            blitflags);

    d2_endframe(d2_handle);     // end of sequence
    d2_startframe(d2_handle);   // start new sequence to force to play previous sequence

    return LLDISPLAY_EXTRA_DRAWING_RUNNING;
}
#endif // USE_D2ENGINE
