/***********************************************************************************************************************
 * Copyright [2015] Renesas Electronics Corporation and/or its licensors. All Rights Reserved.
 * 
 * This file is part of Renesas SynergyTM Software Package (SSP)
 *
 * The contents of this file (the "contents") are proprietary and confidential to Renesas Electronics Corporation
 * and/or its licensors ("Renesas") and subject to statutory and contractual protections.
 *
 * This file is subject to a Renesas SSP license agreement. Unless otherwise agreed in an SSP license agreement with
 * Renesas: 1) you may not use, copy, modify, distribute, display, or perform the contents; 2) you may not use any name
 * or mark of Renesas for advertising or publicity purposes or in connection with your use of the contents; 3) RENESAS
 * MAKES NO WARRANTY OR REPRESENTATIONS ABOUT THE SUITABILITY OF THE CONTENTS FOR ANY PURPOSE; THE CONTENTS ARE PROVIDED
 * "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, AND NON-INFRINGEMENT; AND 4) RENESAS SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, OR
 * CONSEQUENTIAL DAMAGES, INCLUDING DAMAGES RESULTING FROM LOSS OF USE, DATA, OR PROJECTS, WHETHER IN AN ACTION OF
 * CONTRACT OR TORT, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THE CONTENTS. Third-party contents
 * included in this file may be subject to different terms.
 **********************************************************************************************************************/

/**********************************************************************************************************************
 * File Name    : r_dmac.c
 * Description  : DMAC implementation of the transfer interface.
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Includes
 **********************************************************************************************************************/
#include "r_dmac.h"
#include "r_dmac_cfg.h"
#include "hw/hw_dmac_private.h"
#include "r_dmac_private_api.h"

/***********************************************************************************************************************
 * Macro definitions
 **********************************************************************************************************************/
/** Driver ID (DMAC in ASCII) */
#define DMAC_ID (0x444d4143)

/** Macro for error logger. */
#ifndef DMAC_ERROR_RETURN
#define DMAC_ERROR_RETURN(a, err) SSP_ERROR_RETURN((a), (err), "dmac", &g_dmac_version)
#endif

/** Length limited to 1024 transfers for repeat and block mode */
#define DMAC_REPEAT_BLOCK_MAX_LENGTH (0x400)

/***********************************************************************************************************************
 * Typedef definitions
 **********************************************************************************************************************/
/** Internal driver storage for p_callback, used by ISRs. */
typedef struct st_dmac_callback
{
    /** Callback for transfer end interrupt. Set to NULL for no CPU interrupt. */
    void (* p_callback)(transfer_callback_args_t * cb_data);

    /** Placeholder for user data.  Passed to the user p_callback in ::transfer_callback_args_t. */
    void const * p_context;
} dmac_callback_t;

/***********************************************************************************************************************
 * Private function prototypes
 **********************************************************************************************************************/
#if BSP_IRQ_CFG_DMAC0_INT != BSP_IRQ_DISABLED
void dmac0_int_isr (void);
#endif
#if BSP_IRQ_CFG_DMAC1_INT != BSP_IRQ_DISABLED
void dmac1_int_isr (void);
#endif
#if BSP_IRQ_CFG_DMAC2_INT != BSP_IRQ_DISABLED
void dmac2_int_isr (void);
#endif
#if BSP_IRQ_CFG_DMAC3_INT != BSP_IRQ_DISABLED
void dmac3_int_isr (void);
#endif
#if BSP_IRQ_CFG_DMAC4_INT != BSP_IRQ_DISABLED
void dmac4_int_isr (void);
#endif
#if BSP_IRQ_CFG_DMAC5_INT != BSP_IRQ_DISABLED
void dmac5_int_isr (void);
#endif
#if BSP_IRQ_CFG_DMAC6_INT != BSP_IRQ_DISABLED
void dmac6_int_isr (void);
#endif
#if BSP_IRQ_CFG_DMAC7_INT != BSP_IRQ_DISABLED
void dmac7_int_isr (void);
#endif
/***********************************************************************************************************************
 * Private global variables
 **********************************************************************************************************************/
#if defined(__GNUC__)
/* This structure is affected by warnings from the GCC compiler bug https://gcc.gnu.org/bugzilla/show_bug.cgi?id=60784
 * This pragma suppresses the warnings in this structure only, and will be removed when the SSP compiler is updated to
 * v5.3.*/
/*LDRA_INSPECTED 69 S */
#pragma GCC diagnostic ignored "-Wmissing-field-initializers"
#endif
/** Version data structure used by error logger macro. */
static const ssp_version_t g_dmac_version =
{
    .api_version_minor  = TRANSFER_API_VERSION_MINOR,
    .api_version_major  = TRANSFER_API_VERSION_MAJOR,
    .code_version_major = DMAC_CODE_VERSION_MAJOR,
    .code_version_minor = DMAC_CODE_VERSION_MINOR
};
#if defined(__GNUC__)
/* Restore warning settings for 'missing-field-initializers' to as specified on command line. */
/*LDRA_INSPECTED 69 S */
#pragma GCC diagnostic pop
#endif

#if defined(BSP_MCU_GROUP_S7G2)
    #if((BSP_IRQ_CFG_DMAC0_INT != BSP_IRQ_DISABLED) || \
        (BSP_IRQ_CFG_DMAC1_INT != BSP_IRQ_DISABLED) || \
        (BSP_IRQ_CFG_DMAC2_INT != BSP_IRQ_DISABLED) || \
        (BSP_IRQ_CFG_DMAC3_INT != BSP_IRQ_DISABLED) || \
        (BSP_IRQ_CFG_DMAC4_INT != BSP_IRQ_DISABLED) || \
        (BSP_IRQ_CFG_DMAC5_INT != BSP_IRQ_DISABLED) || \
        (BSP_IRQ_CFG_DMAC6_INT != BSP_IRQ_DISABLED) || \
        (BSP_IRQ_CFG_DMAC7_INT != BSP_IRQ_DISABLED)    \
        )
        
        #define DMAC_ISR_ENABLED

    #endif
#elif defined(BSP_MCU_GROUP_S3A7)
    #if((BSP_IRQ_CFG_DMAC0_INT != BSP_IRQ_DISABLED) || \
        (BSP_IRQ_CFG_DMAC1_INT != BSP_IRQ_DISABLED) || \
        (BSP_IRQ_CFG_DMAC2_INT != BSP_IRQ_DISABLED) || \
        (BSP_IRQ_CFG_DMAC3_INT != BSP_IRQ_DISABLED)    \
        )
    
        #define DMAC_ISR_ENABLED
    
    #endif
#endif

#if defined(DMAC_ISR_ENABLED)
/** Callbacks are stored here so they can be accessed by the appropriate ISR */
static dmac_callback_t g_callbacks[DMAC_CHANNEL_MAX] = {{NULL, NULL}};
#endif

/***********************************************************************************************************************
 * Exported global variables
 **********************************************************************************************************************/
/** DMAC implementation of transfer API. */
const transfer_api_t g_transfer_on_dmac =
{
    .open       = R_DMAC_Open,
    .reset      = R_DMAC_Reset,
    .infoGet    = R_DMAC_InfoGet,
    .start      = R_DMAC_Start,
    .stop       = R_DMAC_Stop,
    .enable     = R_DMAC_Enable,
    .disable    = R_DMAC_Disable,
    .close      = R_DMAC_Close,
    .versionGet = R_DMAC_VersionGet
};

/*******************************************************************************************************************//**
 * @addtogroup DMAC
 * @{
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Functions
 **********************************************************************************************************************/

/*******************************************************************************************************************//**
 * @brief  Initialize transfer and enables transfer in ICU. Implements transfer_api_t::open.
 *
 * @retval SSP_SUCCESS              Successful open.  Transfer is configured and will start when trigger occurs.
 * @retval SSP_ERR_ASSERTION        An input parameter is invalid.
 * @retval SSP_ERR_NOT_OPEN         Handle is not initialized.  Call R_DMAC_Open to initialize the control block.
 * @retval SSP_ERR_INTERNAL         All DMAC channels are in use.
 **********************************************************************************************************************/
ssp_err_t R_DMAC_Open      (transfer_ctrl_t         * const p_ctrl,
                            transfer_cfg_t const * const    p_cfg)
{
#if DMAC_CFG_PARAM_CHECKING_ENABLE
    SSP_ASSERT(NULL != p_ctrl);
    SSP_ASSERT(NULL != p_cfg);
    SSP_ASSERT(NULL != p_cfg->p_info);
    SSP_ASSERT(1 != p_cfg->p_info->chain_mode);
    SSP_ASSERT(p_cfg->p_info->mode <= TRANSFER_MODE_BLOCK);
    SSP_ASSERT(p_cfg->p_info->size <= TRANSFER_SIZE_4_BYTE);
    if (TRANSFER_MODE_NORMAL != p_cfg->p_info->mode)
    {
        SSP_ASSERT(p_cfg->p_info->length <= DMAC_REPEAT_BLOCK_MAX_LENGTH);
    }
    DMAC_ERROR_RETURN(p_ctrl->id != DMAC_ID, SSP_ERR_IN_USE);
#endif /* if DMAC_CFG_PARAM_CHECKING_ENABLE */

    /** Select highest priority unused channel, and acquire BSP hardware lock. Return error if all channels
     *  are used. */
    uint8_t ch      = 0;
    while (ch < DMAC_CHANNEL_MAX)
    {
        /* All DMAC channels are listed in order in the bsp_hw_lock_t enum, so adding the channel number offset from
         * the base channel 0 lock yields the channel's lock type. */
        bsp_hw_lock_t lock    = (bsp_hw_lock_t) ((uint8_t) BSP_HW_LOCK_DMAC0 + ch);
        ssp_err_t     bsp_err = R_BSP_HardwareLock(lock);
        if (SSP_SUCCESS == bsp_err)        
        {
            break;
        }
        else
        {
            ch++;
        }
    }

    DMAC_ERROR_RETURN(ch < DMAC_CHANNEL_MAX, SSP_ERR_INTERNAL);

    if (HW_DMAC_StatusGet(p_ctrl->channel))
    {
        return SSP_ERR_IN_USE;
    }

    /** Configure the DMAC according to the flowchart "Activating the DMAC" in chapter 16.3.7 of hardware
     *  manual NoSecurity_r01uh0488ej0040_sc32.pdf. */
    HW_DMAC_PowerOn();
    HW_DMAC_TransferEnableDisable(ch, DMAC_TRANSFER_DISABLE);

    HW_DMAC_DestAddrUpdateModeSet(ch, p_cfg->p_info->dest_addr_mode);
    HW_DMAC_SrcAddrUpdateModeSet(ch, p_cfg->p_info->src_addr_mode);

    /* Extended repeat area not supported. */
    HW_DMAC_DestExtendedRepeatAreaSet(ch, DMAC_EXTENDED_REPEAT_AREA_NONE);
    HW_DMAC_SrcExtendedRepeatAreaSet(ch, DMAC_EXTENDED_REPEAT_AREA_NONE);

    if ((ELC_EVENT_ELC_SOFTWARE_EVENT_0 == p_cfg->activation_source) ||
            (ELC_EVENT_ELC_SOFTWARE_EVENT_1 == p_cfg->activation_source))
    {
        HW_DMAC_SoftwareStartAutoClear(ch, TRANSFER_START_MODE_REPEAT);
        HW_DMAC_SoftwareStop(ch);
        HW_DMAC_TransferRequestSourceSet(ch, DMAC_TRANSFER_REQUEST_SOURCE_SOFTWARE);
    }
    else
    {
        HW_DMAC_TransferRequestSourceSet(ch, DMAC_TRANSFER_REQUEST_SOURCE_INTERRUPT);
    }

    HW_DMAC_TransferSizeSet(ch, p_cfg->p_info->size);
    HW_DMAC_RepeatAreaSet(ch, p_cfg->p_info->repeat_area);
    HW_DMAC_ModeSet(ch, p_cfg->p_info->mode);
    HW_DMAC_SrcStartAddrSet(ch, p_cfg->p_info->p_src);
    HW_DMAC_DestStartAddrSet(ch, p_cfg->p_info->p_dest);
    HW_DMAC_TransferNumberSet(ch, p_cfg->p_info->length);
    if (TRANSFER_MODE_NORMAL == p_cfg->p_info->mode)
    {
        HW_DMAC_TransferReloadSet(ch, 0);
    }
    else
    {
        HW_DMAC_TransferReloadSet(ch, p_cfg->p_info->length);
    }

    HW_DMAC_BlockNumberSet(ch, p_cfg->p_info->num_blocks);

    /** If p_callback is selected, enable interrupts and store the p_callback in an array for access by ISR. */
    if (NULL != p_cfg->p_callback)
    {
        IRQn_Type irq = HW_DMAC_IRQGet(ch);
        HW_DMAC_ICUInterruptClear(irq);
        HW_DMAC_TransferEndInterruptClear(ch);
        NVIC_ClearPendingIRQ(irq);
        NVIC_EnableIRQ(irq);
        HW_DMAC_TransferEndInterruptEnable(ch);
#if defined(DMAC_ISR_ENABLED)
        g_callbacks[ch].p_callback = p_cfg->p_callback;
        g_callbacks[ch].p_context  = p_cfg->p_context;
#endif
    }
    else
    {
        HW_DMAC_TransferEndInterruptDisable(ch);
    }

    /** @note Transfer escape interrupts not supported. */

    /** Update internal variables. */
    p_ctrl->channel = ch;
    p_ctrl->trigger = p_cfg->activation_source;
    p_ctrl->id      = DMAC_ID;

    /** If auto_enable is true, enable transfer and ELC events if software start is used. */
    ssp_err_t err = SSP_SUCCESS;
    if (p_cfg->auto_enable)
    {
        err = R_DMAC_Enable(p_ctrl);
    }
    HW_DMAC_Enable();

    DMAC_ERROR_RETURN(SSP_SUCCESS == err, SSP_ERR_NOT_ENABLED);

    return SSP_SUCCESS;
} /* End of function R_DMAC_Open */

/*******************************************************************************************************************//**
 * @brief  Reset transfer source, destination, and number of transfers.
 *
 * @retval SSP_SUCCESS              Transfer reset successfully.
 * @retval SSP_ERR_ASSERTION        An input parameter is invalid.
 * @retval SSP_ERR_NOT_OPEN      Handle is not initialized.  Call R_DMAC_Open to initialize the control block.
 **********************************************************************************************************************/
ssp_err_t R_DMAC_Reset (transfer_ctrl_t         * const   p_ctrl,
                        void const * volatile             p_src,
                        void                   * volatile p_dest,
                        uint16_t const                    num_transfers)
{
#if DMAC_CFG_PARAM_CHECKING_ENABLE
    SSP_ASSERT(NULL != p_ctrl);
    SSP_ASSERT(0 != num_transfers);
    DMAC_ERROR_RETURN(p_ctrl->id == DMAC_ID, SSP_ERR_NOT_OPEN);
#endif

    if (HW_DMAC_StatusGet(p_ctrl->channel))
    {
        return SSP_ERR_IN_USE;
    }

    HW_DMAC_TransferEnableDisable(p_ctrl->channel, DMAC_TRANSFER_DISABLE);
    if (NULL != p_src)
    {
        HW_DMAC_SrcStartAddrSet(p_ctrl->channel, p_src);
    }

    if (NULL != p_dest)
    {
        HW_DMAC_DestStartAddrSet(p_ctrl->channel, p_dest);
    }

    if ((TRANSFER_MODE_BLOCK == HW_DMAC_ModeGet(p_ctrl->channel)) ||
       (TRANSFER_MODE_REPEAT == HW_DMAC_ModeGet(p_ctrl->channel)))
    {
        HW_DMAC_BlockNumberSet(p_ctrl->channel, num_transfers);
    }
    else
    {
        HW_DMAC_TransferNumberSet(p_ctrl->channel, num_transfers);
    }

    /** Enables transfers on this activation source. */
    ssp_err_t err = R_DMAC_Enable(p_ctrl);
    DMAC_ERROR_RETURN(SSP_SUCCESS == err, SSP_ERR_NOT_ENABLED);

    return SSP_SUCCESS;
} /* End of function R_DMAC_Reset */

/*******************************************************************************************************************//**
 * @brief  Start transfer. Implements transfer_api_t::start.
 *
 * @retval SSP_SUCCESS              Transfer started written successfully.
 * @retval SSP_ERR_ASSERTION        An input parameter is invalid.
 * @retval SSP_ERR_NOT_OPEN         Handle is not initialized.  Call R_DMAC_Open to initialize the control block.
 * @retval SSP_ERR_UNSUPPORTED      Handle was not configured for software activation.
 **********************************************************************************************************************/
ssp_err_t R_DMAC_Start     (transfer_ctrl_t         * const p_ctrl,
                            transfer_start_mode_t           mode)
{
#if DMAC_CFG_PARAM_CHECKING_ENABLE
    SSP_ASSERT(NULL != p_ctrl);
    SSP_ASSERT(mode < 2);
    DMAC_ERROR_RETURN(p_ctrl->id == DMAC_ID, SSP_ERR_NOT_OPEN);
#endif

    if ((ELC_EVENT_ELC_SOFTWARE_EVENT_0 == p_ctrl->trigger) ||
            (ELC_EVENT_ELC_SOFTWARE_EVENT_1 == p_ctrl->trigger))
    {
        /** Set autoclear bit and software start bit. */
        HW_DMAC_SoftwareStartAutoClear(p_ctrl->channel, mode);
        HW_DMAC_SoftwareStart(p_ctrl->channel);
    }
    else
    {
        /** Software start is unsupported if channel was configured with an interrupt activation source. */
        DMAC_ERROR_RETURN(false, SSP_ERR_UNSUPPORTED);
    }

    return SSP_SUCCESS;
} /* End of function R_DMAC_Start */

/*******************************************************************************************************************//**
 * @brief  Stop transfer. Implements transfer_api_t::stop.
 *
 * @retval SSP_SUCCESS              Transfer stopped written successfully.
 * @retval SSP_ERR_ASSERTION        An input parameter is invalid.
 * @retval SSP_ERR_NOT_OPEN         Handle is not initialized.  Call R_DMAC_Open to initialize the control block.
 **********************************************************************************************************************/
ssp_err_t R_DMAC_Stop       (transfer_ctrl_t         * const p_ctrl)
{
#if DMAC_CFG_PARAM_CHECKING_ENABLE
    SSP_ASSERT(NULL != p_ctrl);
    DMAC_ERROR_RETURN(p_ctrl->id == DMAC_ID, SSP_ERR_NOT_OPEN);
#endif

    /** Reset auto clear bit and clear software transfer request. */
    HW_DMAC_SoftwareStartAutoClear(p_ctrl->channel, TRANSFER_START_MODE_SINGLE);
    HW_DMAC_SoftwareStop(p_ctrl->channel);

    return SSP_SUCCESS;
} /* End of function R_DMAC_Stop */

/*******************************************************************************************************************//**
 * @brief  Enable transfer. Implements transfer_api_t::enable.
 *
 * @retval SSP_SUCCESS              Counter value written successfully.
 * @retval SSP_ERR_ASSERTION        An input parameter is invalid.
 * @retval SSP_ERR_NOT_OPEN         Handle is not initialized.  Call R_DTC_Open to initialize the control block.
 **********************************************************************************************************************/
ssp_err_t R_DMAC_Enable (transfer_ctrl_t         * const p_ctrl)
{
#if DMAC_CFG_PARAM_CHECKING_ENABLE
    SSP_ASSERT(NULL != p_ctrl);
    SSP_ASSERT(NULL != HW_DMAC_DestStartAddrGet(p_ctrl->channel));
    SSP_ASSERT(NULL != HW_DMAC_SrcStartAddrGet(p_ctrl->channel));
    DMAC_ERROR_RETURN(p_ctrl->id == DMAC_ID, SSP_ERR_NOT_OPEN);
#endif

    /** Enable transfer. */
    HW_ICU_DmacEnable(p_ctrl->channel, p_ctrl->trigger);
    HW_DMAC_TransferEnableDisable(p_ctrl->channel, DMAC_TRANSFER_ENABLE);

    return SSP_SUCCESS;
} /* End of function R_DTC_Enable */

/*******************************************************************************************************************//**
 * @brief  Disable transfer. Implements transfer_api_t::disable.
 *
 * @retval SSP_SUCCESS              Counter value written successfully.
 * @retval SSP_ERR_ASSERTION        An input parameter is invalid.
 * @retval SSP_ERR_NOT_OPEN         Handle is not initialized.  Call R_DTC_Open to initialize the control block.
 **********************************************************************************************************************/
ssp_err_t R_DMAC_Disable (transfer_ctrl_t         * const p_ctrl)
{
#if DMAC_CFG_PARAM_CHECKING_ENABLE
    SSP_ASSERT(NULL != p_ctrl);
    DMAC_ERROR_RETURN(p_ctrl->id == DMAC_ID, SSP_ERR_NOT_OPEN);
#endif

    /** Disable transfer. */
    /* 0 is not a valid ELC event number, so this will not cause a different event to start a transfer. */
    HW_ICU_DmacEnable(p_ctrl->channel, (elc_event_t) 0);
    HW_DMAC_TransferEnableDisable(p_ctrl->channel, DMAC_TRANSFER_DISABLE);

    return SSP_SUCCESS;
} /* End of function R_DTC_Disable */

/*******************************************************************************************************************//**
 * @brief  Set driver specific information in provided pointer. Implements transfer_api_t::infoGet.
 *
 * @retval SSP_SUCCESS              Counter value written successfully.
 * @retval SSP_ERR_ASSERTION        An input parameter is invalid.
 **********************************************************************************************************************/
ssp_err_t R_DMAC_InfoGet (transfer_ctrl_t       * const p_ctrl,
                          transfer_properties_t * const p_info)
{
#if DMAC_CFG_PARAM_CHECKING_ENABLE
    SSP_ASSERT(NULL != p_ctrl);
    SSP_ASSERT(NULL != p_info);
#endif

    /** If a transfer is active, store it in p_in_progress. */
    p_info->in_progress = HW_DMAC_StatusGet(p_ctrl->channel);

    /** Store maximum transfer length. */
    transfer_mode_t mode = HW_DMAC_ModeGet(p_ctrl->channel);
    if (TRANSFER_MODE_NORMAL == mode)
    {
        p_info->transfer_length_max = DMAC_NORMAL_MAX_LENGTH;
    }
    else
    {
        p_info->transfer_length_max = DMAC_REPEAT_BLOCK_MAX_LENGTH;
    }
    /** Store remaining transfer length. */
    if (TRANSFER_MODE_BLOCK == mode)
    {
    	p_info->transfer_length_remaining = HW_DMAC_BlockNumberGet(p_ctrl->channel);
    }
    else if(TRANSFER_MODE_REPEAT == mode)
    {

    	p_info->transfer_length_remaining = HW_DMAC_BlockNumberGet(p_ctrl->channel);
    }
    else
    {
    	p_info->transfer_length_remaining = HW_DMAC_TransferNumberGet(p_ctrl->channel);
    }

    return SSP_SUCCESS;
} /* End of function R_DMAC_StatusGet */

/*******************************************************************************************************************//**
 * @brief      Disable transfer in the ICU, then clear transfer data from the DTC vector table. Implements
 *             transfer_api_t::close.
 *
 * @retval SSP_SUCCESS           Successful close.
 * @retval SSP_ERR_ASSERTION     An input parameter is invalid.
 * @retval SSP_ERR_NOT_OPEN      Handle is not initialized.  Call R_DMAC_Open to initialize the control block.
 **********************************************************************************************************************/
ssp_err_t R_DMAC_Close     (transfer_ctrl_t         * const p_ctrl)
{
#if DMAC_CFG_PARAM_CHECKING_ENABLE
    SSP_ASSERT(NULL != p_ctrl);
    DMAC_ERROR_RETURN(p_ctrl->id == DMAC_ID, SSP_ERR_NOT_OPEN);
#endif

    if (HW_DMAC_StatusGet(p_ctrl->channel))
    {
        return SSP_ERR_IN_USE;
    }

    /** Disable DMAC transfers, disable DMAC interrupts, and remove DMAC trigger from ICU register. */
    /* 0 is not a valid ELC event number, so this will not cause a different event to start a transfer. */
    HW_ICU_DmacEnable(p_ctrl->channel, (elc_event_t) 0);
    HW_DMAC_TransferEnableDisable(p_ctrl->channel, DMAC_TRANSFER_DISABLE);
    IRQn_Type irq = HW_DMAC_IRQGet(p_ctrl->channel);
    NVIC_DisableIRQ(irq);

    /** Clear internal driver data */
#if defined(DMAC_ISR_ENABLED)
    g_callbacks[p_ctrl->channel].p_callback = NULL;
    g_callbacks[p_ctrl->channel].p_context  = 0;
#endif

    /** Release BSP hardware lock on this channel */
    /* All DMAC channels are listed in order in the bsp_hw_lock_t enum, so adding the channel number offset from
     * the base channel 0 lock yields the channel's lock type. */
    bsp_hw_lock_t lock = (bsp_hw_lock_t) ((uint8_t) BSP_HW_LOCK_DMAC0 + p_ctrl->channel);
    R_BSP_HardwareUnlock(lock);

    /** Clear ID so control block can be reused. */
    p_ctrl->id = 0U;

    return SSP_SUCCESS;
} /* End of function R_DMAC_Close */

/*******************************************************************************************************************//**
 * @brief      Set driver version based on compile time macros.
 *
 * @retval SSP_SUCCESS              Successful close.
 * @retval SSP_ERR_ASSERTION        An input parameter is invalid.
 **********************************************************************************************************************/
ssp_err_t R_DMAC_VersionGet (ssp_version_t     * const p_version)
{
#if DMAC_CFG_PARAM_CHECKING_ENABLE
    /** Verify parameters are valid */
    SSP_ASSERT(NULL != p_version);
#endif

    p_version->version_id = g_dmac_version.version_id;

    return SSP_SUCCESS;
} /* End of function R_DMAC_VersionGet */

/*******************************************************************************************************************//**
 * @} (end addtogroup DMAC)
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Private Functions
 * @brief      Channel 0 ISR.
 *
 * Saves p_context if RTOS is used, clears the interrupt flag, calls p_callback if one was provided in the open
 * function, and restores p_context if RTOS is used.
 **********************************************************************************************************************/
#ifdef BSP_IRQ_CFG_DMAC0_INT
#if BSP_IRQ_CFG_DMAC0_INT != BSP_IRQ_DISABLED
void dmac0_int_isr (void)
{
    /* Save context if RTOS is used */
    SF_CONTEXT_SAVE

    /** Clear pending IRQ to make sure it doesn't fire again after exiting */
    R_BSP_IrqStatusClear (DMAC0_INT_IRQn);

    /** Call user p_callback */
    if (NULL != g_callbacks[0].p_callback)
    {
        transfer_callback_args_t args;
        args.p_context = g_callbacks[0].p_context;
        g_callbacks[0].p_callback(&args);
    }

    /* Restore context if RTOS is used */
    SF_CONTEXT_RESTORE
} /* End of function dmac0_int_isr */
#endif /* if BSP_IRQ_CFG_DMAC0_INT != BSP_IRQ_DISABLED */
#endif

/*******************************************************************************************************************//**
 * @brief      Channel 1 ISR.
 *
 * Saves p_context if RTOS is used, clears the interrupt flag, calls p_callback if one was provided in the open
 * function, and restores p_context if RTOS is used.
 **********************************************************************************************************************/
#ifdef BSP_IRQ_CFG_DMAC1_INT
#if BSP_IRQ_CFG_DMAC1_INT != BSP_IRQ_DISABLED
void dmac1_int_isr (void)
{
    /* Save context if RTOS is used */
    SF_CONTEXT_SAVE

    /** Clear pending IRQ to make sure it doesn't fire again after exiting */
    R_BSP_IrqStatusClear (DMAC1_INT_IRQn);

    /** Call user p_callback */
    if (NULL != g_callbacks[1].p_callback)
    {
        transfer_callback_args_t args;
        args.p_context = g_callbacks[1].p_context;
        g_callbacks[1].p_callback(&args);
    }

    /* Restore context if RTOS is used */
    SF_CONTEXT_RESTORE
} /* End of function dmac1_int_isr */
#endif /* if BSP_IRQ_CFG_DMAC1_INT != BSP_IRQ_DISABLED */
#endif

/*******************************************************************************************************************//**
 * @brief      Channel 2 ISR.
 *
 * Saves p_context if RTOS is used, clears the interrupt flag, calls p_callback if one was provided in the open
 * function, and restores p_context if RTOS is used.
 **********************************************************************************************************************/
#ifdef BSP_IRQ_CFG_DMAC2_INT
#if BSP_IRQ_CFG_DMAC2_INT != BSP_IRQ_DISABLED
void dmac2_int_isr (void)
{
    /* Save context if RTOS is used */
    SF_CONTEXT_SAVE

    /** Clear pending IRQ to make sure it doesn't fire again after exiting */
    R_BSP_IrqStatusClear (DMAC2_INT_IRQn);

    /** Call user p_callback */
    if (NULL != g_callbacks[2].p_callback)
    {
        transfer_callback_args_t args;
        args.p_context = g_callbacks[2].p_context;
        g_callbacks[2].p_callback(&args);
    }

    /* Restore context if RTOS is used */
    SF_CONTEXT_RESTORE
} /* End of function dmac2_int_isr */
#endif /* if BSP_IRQ_CFG_DMAC2_INT != BSP_IRQ_DISABLED */
#endif

/*******************************************************************************************************************//**
 * @brief      Channel 3 ISR.
 *
 * Saves p_context if RTOS is used, clears the interrupt flag, calls p_callback if one was provided in the open
 * function, and restores p_context if RTOS is used.
 **********************************************************************************************************************/
#ifdef BSP_IRQ_CFG_DMAC3_INT
#if BSP_IRQ_CFG_DMAC3_INT != BSP_IRQ_DISABLED
void dmac3_int_isr (void)
{
    /* Save context if RTOS is used */
    SF_CONTEXT_SAVE

    /** Clear pending IRQ to make sure it doesn't fire again after exiting */
    R_BSP_IrqStatusClear (DMAC3_INT_IRQn);

    /** Call user p_callback */
    if (NULL != g_callbacks[3].p_callback)
    {
        transfer_callback_args_t args;
        args.p_context = g_callbacks[3].p_context;
        g_callbacks[3].p_callback(&args);
    }

    /* Restore context if RTOS is used */
    SF_CONTEXT_RESTORE
} /* End of function dmac3_int_isr */
#endif /* if BSP_IRQ_CFG_DMAC3_INT != BSP_IRQ_DISABLED */
#endif

/*******************************************************************************************************************//**
 * @brief      Channel 4 ISR.
 *
 * Saves p_context if RTOS is used, clears the interrupt flag, calls p_callback if one was provided in the open
 * function, and restores p_context if RTOS is used.
 **********************************************************************************************************************/
#ifdef BSP_IRQ_CFG_DMAC4_INT
#if BSP_IRQ_CFG_DMAC4_INT != BSP_IRQ_DISABLED
void dmac4_int_isr (void)
{
    /* Save context if RTOS is used */
    SF_CONTEXT_SAVE

    /** Clear pending IRQ to make sure it doesn't fire again after exiting */
    R_BSP_IrqStatusClear (DMAC4_INT_IRQn);

    /** Call user p_callback */
    if (NULL != g_callbacks[4].p_callback)
    {
        transfer_callback_args_t args;
        args.p_context = g_callbacks[4].p_context;
        g_callbacks[4].p_callback(&args);
    }

    /* Restore context if RTOS is used */
    SF_CONTEXT_RESTORE
} /* End of function dmac4_int_isr */
#endif /* if BSP_IRQ_CFG_DMAC4_INT != BSP_IRQ_DISABLED */
#endif

/*******************************************************************************************************************//**
 * @brief      Channel 5 ISR.
 *
 * Saves p_context if RTOS is used, clears the interrupt flag, calls p_callback if one was provided in the open
 * function, and restores p_context if RTOS is used.
 **********************************************************************************************************************/
#ifdef BSP_IRQ_CFG_DMAC5_INT
#if BSP_IRQ_CFG_DMAC5_INT != BSP_IRQ_DISABLED
void dmac5_int_isr (void)
{
    /* Save context if RTOS is used */
    SF_CONTEXT_SAVE

    /** Clear pending IRQ to make sure it doesn't fire again after exiting */
    R_BSP_IrqStatusClear (DMAC5_INT_IRQn);

    /** Call user p_callback */
    if (NULL != g_callbacks[5].p_callback)
    {
        transfer_callback_args_t args;
        args.p_context = g_callbacks[5].p_context;
        g_callbacks[5].p_callback(&args);
    }

    /* Restore context if RTOS is used */
    SF_CONTEXT_RESTORE
} /* End of function dmac5_int_isr */
#endif /* if BSP_IRQ_CFG_DMAC5_INT != BSP_IRQ_DISABLED */
#endif

/*******************************************************************************************************************//**
 * @brief      Channel 6 ISR.
 *
 * Saves p_context if RTOS is used, clears the interrupt flag, calls p_callback if one was provided in the open
 * function, and restores p_context if RTOS is used.
 **********************************************************************************************************************/
#ifdef BSP_IRQ_CFG_DMAC6_INT
#if BSP_IRQ_CFG_DMAC6_INT != BSP_IRQ_DISABLED
void dmac6_int_isr (void)
{
    /* Save context if RTOS is used */
    SF_CONTEXT_SAVE

    /** Clear pending IRQ to make sure it doesn't fire again after exiting */
    R_BSP_IrqStatusClear (DMAC6_INT_IRQn);

    /** Call user p_callback */
    if (NULL != g_callbacks[6].p_callback)
    {
        transfer_callback_args_t args;
        args.p_context = g_callbacks[6].p_context;
        g_callbacks[6].p_callback(&args);
    }

    /* Restore context if RTOS is used */
    SF_CONTEXT_RESTORE
} /* End of function dmac6_int_isr */
#endif /* if BSP_IRQ_CFG_DMAC6_INT != BSP_IRQ_DISABLED */
#endif

/*******************************************************************************************************************//**
 * @brief      Channel 7 ISR.
 *
 * Saves p_context if RTOS is used, clears the interrupt flag, calls p_callback if one was provided in the open
 * function, and restores p_context if RTOS is used.
 **********************************************************************************************************************/
#ifdef BSP_IRQ_CFG_DMAC7_INT
#if BSP_IRQ_CFG_DMAC7_INT != BSP_IRQ_DISABLED
void dmac7_int_isr (void)
{
    /* Save context if RTOS is used */
    SF_CONTEXT_SAVE

    /** Clear pending IRQ to make sure it doesn't fire again after exiting */
    R_BSP_IrqStatusClear (DMAC7_INT_IRQn);

    /** Call user p_callback */
    if (NULL != g_callbacks[7].p_callback)
    {
        transfer_callback_args_t args;
        args.p_context = g_callbacks[7].p_context;
        g_callbacks[7].p_callback(&args);
    }

    /* Restore context if RTOS is used */
    SF_CONTEXT_RESTORE
} /* End of function dmac7_int_isr */
#endif /* if BSP_IRQ_CFG_DMAC7_INT != BSP_IRQ_DISABLED */
#endif

