/***********************************************************************************************************************
 * Copyright [2015] Renesas Electronics Corporation and/or its licensors. All Rights Reserved.
 * 
 * This file is part of Renesas SynergyTM Software Package (SSP)
 *
 * The contents of this file (the "contents") are proprietary and confidential to Renesas Electronics Corporation
 * and/or its licensors ("Renesas") and subject to statutory and contractual protections.
 *
 * This file is subject to a Renesas SSP license agreement. Unless otherwise agreed in an SSP license agreement with
 * Renesas: 1) you may not use, copy, modify, distribute, display, or perform the contents; 2) you may not use any name
 * or mark of Renesas for advertising or publicity purposes or in connection with your use of the contents; 3) RENESAS
 * MAKES NO WARRANTY OR REPRESENTATIONS ABOUT THE SUITABILITY OF THE CONTENTS FOR ANY PURPOSE; THE CONTENTS ARE PROVIDED
 * "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, AND NON-INFRINGEMENT; AND 4) RENESAS SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, OR
 * CONSEQUENTIAL DAMAGES, INCLUDING DAMAGES RESULTING FROM LOSS OF USE, DATA, OR PROJECTS, WHETHER IN AN ACTION OF
 * CONTRACT OR TORT, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THE CONTENTS. Third-party contents
 * included in this file may be subject to different terms.
 **********************************************************************************************************************/

/**********************************************************************************************************************
 * File Name    : hw_gpt_common.h
 * Description  : Lower level driver (register access) functions for GPT.
 **********************************************************************************************************************/

#ifndef HW_GPT_COMMON_H
#define HW_GPT_COMMON_H

/**********************************************************************************************************************
 * Includes
 **********************************************************************************************************************/
#include "bsp_api.h"
#ifdef BSP_MCU_GROUP_S7G2
#include "..\..\..\src\driver\r_gpt\hw\target\s7g2\hw_gpt_s7g2.h"
#endif

#ifdef BSP_MCU_GROUP_S3A7
#include "..\..\..\src\driver\r_gpt\hw\target\s3a7\hw_gpt_s3a7.h"
#endif

#ifdef BSP_MCU_GROUP_S124
#include "..\..\..\src\driver\r_gpt\hw\target\s124\hw_gpt_s124.h"
#endif
/**********************************************************************************************************************
 * Macro Definitions
 **********************************************************************************************************************/
#ifdef BSP_MCU_GROUP_S7G2
#define HW_GPT_MSTPD5_MAX_CH   (7)
#endif

#ifdef BSP_MCU_GROUP_S3A7
#define HW_GPT_MSTPD5_MAX_CH   (3)
#endif

#ifdef BSP_MCU_GROUP_S124
#define HW_GPT_MSTPD5_MAX_CH   (0)
#endif

/***********************************************************************************************************************
 * Private global variables
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Private Functions
 **********************************************************************************************************************/

/*******************************************************************************************************************//**
 * Powers on the GPT block.  Power control is divided into 2 blocks.
 * @param  channel    The channel corresponds to the hardware channel number.
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_PowerOn (uint8_t const channel)
{
    R_BSP_RegisterProtectDisable(BSP_REG_PROTECT_OM_LPC_BATT);
    if (channel <= HW_GPT_MSTPD5_MAX_CH)
    {
        R_MSTP->MSTPCRD_b.MSTPD5 = 0;
    }
    else
    {
        R_MSTP->MSTPCRD_b.MSTPD6 = 0;
    }
    R_BSP_RegisterProtectEnable(BSP_REG_PROTECT_OM_LPC_BATT);
}

/*******************************************************************************************************************//**
 * Powers off the GPT block.  Power control is divided into 2 blocks.
 * @param  channel    The channel corresponds to the hardware channel number.
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_PowerOff (uint8_t const channel)
{
    R_BSP_RegisterProtectDisable(BSP_REG_PROTECT_OM_LPC_BATT);
    if (channel <= HW_GPT_MSTPD5_MAX_CH)
    {
        R_MSTP->MSTPCRD_b.MSTPD5 = 1;
    }
    else
    {
        R_MSTP->MSTPCRD_b.MSTPD6 = 1;
    }
    R_BSP_RegisterProtectEnable(BSP_REG_PROTECT_OM_LPC_BATT);
}

/*******************************************************************************************************************//**
 * Sets the timer operational mode.
 * @param  channel    The channel corresponds to the hardware channel number.
 * @param  mode  Select mode to set.  See ::gpt_mode_t.
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_ModeSet (uint8_t const channel, gpt_mode_t mode)
{
    /* Set mode */
    GPT_CH(channel).GTCR_b.MD = mode;
}

/*******************************************************************************************************************//**
 * Sets the timer cycle (compare match value).  The timer cycle, along with the PCLK frequency and the GPT PCLK divisor
 * control the timer period.
 * @param  channel           The channel corresponds to the hardware channel number.
 * @param  timer_cycle  Any number from 0 to 0xFFFFFFFF.  The resulting delay period can be calculated according to:
 *                           \f$period=\frac{(timer_cycle+1) \times GPTPCLKDivisor}{PCLKFrequencyHz}\f$
 *                           where GPTPCLKDivisor is any of the following: 1, 4, 16, 64, 256, or 1024
 *                           (see ::HW_GPT_DivisorSet)
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_TimerCycleSet (uint8_t const channel, uint32_t timer_cycle)
{
    /* Set delay */
    GPT_CH(channel).GTPR = timer_cycle;
}

/*******************************************************************************************************************//**
 * Returns the timer cycle (compare match value).  The timer cycle, along with the PCLK frequency and the GPT PCLK
 * divisor control the timer period.
 * @param   channel           The channel corresponds to the hardware channel number.
 * @return  The timer cycle value.  See ::HW_GPT_TimerCycleSet for relationship to timer period.
 **********************************************************************************************************************/
__STATIC_INLINE uint32_t HW_GPT_TimerCycleGet (uint8_t const channel)
{
    return GPT_CH(channel).GTPR;
}

/*******************************************************************************************************************//**
 * Sets the GPT PCLK divisor.  The GPT PCLK divisor, along with the PCLK frequency and the timer cycle control the
 * timer period.
 * @param   channel           The channel corresponds to the hardware channel number.
 * @param   pclk_divisor See ::gpt_pclk_div_t for available divisors.
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_DivisorSet (uint8_t const channel, gpt_pclk_div_t pclk_divisor)
{
    GPT_CH(channel).GTCR_b.TPCS = pclk_divisor;
}

/*******************************************************************************************************************//**
 * Gets the GPT PCLK divisor.  The GPT PCLK divisor, along with the PCLK frequency and the timer cycle control the
 * timer period.
 * @param   channel           The channel corresponds to the hardware channel number.
 * @retval  PCLK divisor value. See ::gpt_pclk_div_t for the detail.
 **********************************************************************************************************************/
__STATIC_INLINE gpt_pclk_div_t HW_GPT_DivisorGet (uint8_t const channel)
{
    return (gpt_pclk_div_t) GPT_CH(channel).GTCR_b.TPCS;
}

/*******************************************************************************************************************//**
 * Starts the counter.
 * @param   channel           The channel corresponds to the hardware channel number.
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_CounterStartStop (uint8_t const channel, gpt_start_status_t start_stop)
{
    GPT_CH(channel).GTCR_b.CST = start_stop;
}

/*******************************************************************************************************************//**
 * Sets the counter value.
 * @param  channel           The channel corresponds to the hardware channel number.
 * @param  value        Any number from 0 to 0xFFFFFFFF.
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_CounterSet (uint8_t const channel, uint32_t value)
{
    /* Set counter value */
    GPT_CH(channel).GTCNT = value;
}


/*******************************************************************************************************************//**
 * Returns the current counter value.
 * @param   channel           The channel corresponds to the hardware channel number.
 * @return  Current counter value in the range 0 to 0xFFFFFFFF.
 **********************************************************************************************************************/
__STATIC_INLINE uint32_t HW_GPT_CounterGet (uint8_t const channel)
{
    /* Get counter value */
    return GPT_CH(channel).GTCNT;
}

/*******************************************************************************************************************//**
 * Sets the direction the counter counts.
 * @param  channel           The channel corresponds to the hardware channel number.
 * @param  dir          See ::gpt_dir_t for available values.
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_DirectionSet (uint8_t const channel, gpt_dir_t dir)
{
    /* Force setting */
    GPT_CH(channel).GTUDDTYC_b.UDF = 1;

    /* Start timer */
    GPT_CH(channel).GTUDDTYC_b.UD = dir;

    /* Clear forcing */
    GPT_CH(channel).GTUDDTYC_b.UDF = 0;
}

/*******************************************************************************************************************//**
 * Sets the direction the counter counts.
 * @param  channel           The channel corresponds to the hardware channel number.
 * @retval Count Direction Setting(GPT_DIR_COUNT_DOWN or GPT_DIR_COUNT_UP)
 **********************************************************************************************************************/
__STATIC_INLINE timer_direction_t HW_GPT_DirectionGet (uint8_t const channel)
{
    /* Read cound direction setting */
    return (timer_direction_t) GPT_CH(channel).GTUDDTYC_b.UD;
}

/*******************************************************************************************************************//**
 * Enables interrupts for requested triggers.
 * @param  channel           The channel corresponds to the hardware channel number.
 * @param  trigger      See ::gpt_int_trig_t for available values.
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_InterruptEnable (uint8_t const channel, gpt_int_trig_t trigger)
{
    GPT_CH(channel).GTINTAD_b.GTINTPR = trigger;
}

/*******************************************************************************************************************//**
 * Disables interrupts for specified channel.
 * @param  channel           The channel corresponds to the hardware channel number.
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_InterruptDisable (uint8_t const channel)
{
    GPT_CH(channel).GTINTAD_b.GTINTPR = GPT_INT_TYPE_NONE;
}

/*******************************************************************************************************************//**
 * Returns the value of the overflow interrupt flag.
 * @param   channel           The channel corresponds to the hardware channel number.
 * @return  Overflow interrupt flag value.
 **********************************************************************************************************************/
__STATIC_INLINE bool HW_GPT_InterruptQuery (uint8_t const channel)
{
    return GPT_CH(channel).GTST_b.TCPFO;
}

/*******************************************************************************************************************//**
 * Clears interrupts for specified channel.
 * @param  channel           The channel corresponds to the hardware channel number.
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_InterruptClear (uint8_t const channel)
{
    GPT_CH(channel).GTST_b.TCPFO = 0;
}

/*******************************************************************************************************************//**
 * Clears interrupts for specified channel at ICU.
 * @param  irq_num       The channel corresponds to the hardware channel number.
 **********************************************************************************************************************/
__STATIC_INLINE void HW_ICU_InterruptClear (IRQn_Type irq_num)
{
    /* Clear the interrupt request flag */
    R_ICU->IELSRn_b[irq_num].IR = 0;
}

/*******************************************************************************************************************//**
 * Returns the value of the start bit, indicating if the counter is counting or not.
 * @param   channel           The channel corresponds to the hardware channel number.
 * @return  Start bit value
 **********************************************************************************************************************/
__STATIC_INLINE gpt_start_status_t HW_GPT_CounterStartBitGet (uint8_t const channel)
{
    return (gpt_start_status_t) GPT_CH(channel).GTCR_b.CST;
}

/*******************************************************************************************************************//**
 * Returns the IRQ number required for NVIC calls.
 * @param   channel           The channel corresponds to the hardware channel number.
 * @return  The IRQ number required for NVIC calls.
 **********************************************************************************************************************/
__STATIC_INLINE IRQn_Type HW_GPT_IRQGet (uint8_t const channel)
{
    return irq_lookup[channel];
}

/*******************************************************************************************************************//**
 * Returns the IRQ number required for NVIC calls.
 * @param   channel           The channel corresponds to the hardware channel number.
 * @return  The IRQ number required for NVIC calls.
 **********************************************************************************************************************/
__STATIC_INLINE IRQn_Type HW_GPT_CaptureA_IRQGet(uint8_t const channel)
{
    return irq_captureA_lookup[channel];
}

/*******************************************************************************************************************//**
 * Lookup base address of input/output control register for specified channel and pin.
 * @param   channel           The channel corresponds to the hardware channel number.
 * @param   gtioc        Specify which output pin to use.
 * @param[out] pp_reg    Pointer to pointer to GTIORH or GTIORL.
 **********************************************************************************************************************/
__STATIC_INLINE gtior_t * HW_GPT_GTIOCRegLookup (uint8_t const channel, gpt_gtioc_t gtioc)
{
    if (GPT_GTIOCA == gtioc)
    {
        return ((gtior_t *) &GPT_CH(channel).GTIOR) + 1;
    }
    else
    {
        return ((gtior_t *) &GPT_CH(channel).GTIOR);
    }
}

/*******************************************************************************************************************//**
 * Set level to output when counter is stopped.
 * @param   p_reg        Pointer to input/output control register
 * @param   level        Level to output when counter is stopped.
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_GTIOCPinLevelStoppedSet (gtior_t * const p_reg, gpt_pin_level_t const level)
{
    p_reg->GTIOR_b.OADFLT = level;
}

/*******************************************************************************************************************//**
 * Enable output to pin.
 * @param   p_reg        Pointer to input/output control register
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_GTIOCPinOutputEnable (gtior_t * const p_reg, bool enable)
{
    p_reg->GTIOR_b.OAE = enable;
}

/*******************************************************************************************************************//**
 * Disable output to pin.
 * @param   p_reg        Pointer to input/output control register
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_GTIOCPinOutputDisable (gtior_t * const p_reg)
{
    p_reg->GTIOR_b.OAE = 0;
}

/*******************************************************************************************************************//**
 * Set how to change output at cycle end.
 * @param   channel           The channel corresponds to the hardware channel number.
 * @param   mode              Level to output at cycle end
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_GTIOCCycleEndOutputSet (gtior_t * const p_reg, gpt_output_t const output)
{
    p_reg->GTIOR_b.GTIOCE = output;
}

/*******************************************************************************************************************//**
 * Set compare match mode
 * @param   p_reg        Pointer to input/output control register
 * @param   value        Compare match value to set
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_DutyCycleModeSet (uint8_t const channel, gpt_gtioc_t gtio, gpt_duty_cycle_mode_t mode)
{
    if (GPT_GTIOCA == gtio)
    {
        GPT_CH(channel).GTUDDTYC_b.OADTY = mode;
    }
    else if (GPT_GTIOCB == gtio)
    {
        GPT_CH(channel).GTUDDTYC_b.OBDTY = mode;
    }
}

/*******************************************************************************************************************//**
 * Enable buffer.
 * @param   p_reg        Pointer to input/output control register
 * @param   value        Compare match value to set
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_SingleBufferEnable (uint8_t const channel, gpt_gtioc_t gtio)
{
    if (GPT_GTIOCA == gtio)
    {
        /* GTCCRA buffer enable. */
        GPT_CH(channel).GTBER_b.CCRA = 1;
    }
    else
    {
        /* GTCCRB buffer enable. */
        GPT_CH(channel).GTBER_b.CCRB = 1;
    }
}

/*******************************************************************************************************************//**
 * Set compare match value
 * @param   p_reg        Pointer to input/output control register
 * @param   value        Compare match value to set
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_InitialCompareMatchSet (uint8_t const channel, gpt_gtioc_t gtio, uint32_t const value)
{
    if (GPT_GTIOCA == gtio)
    {
        /* GTCCRA buffer register. */
        GPT_CH(channel).GTCCRA = value;
    }
    else
    {
        /* GTCCRB buffer register. */
        GPT_CH(channel).GTCCRB = value;
    }
}

/*******************************************************************************************************************//**
 * Set compare match value
 * @param   p_reg        Pointer to input/output control register
 * @param   value        Compare match value to set
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_CompareMatchSet (uint8_t const channel, gpt_gtioc_t gtio, uint32_t const value)
{
    if (GPT_GTIOCA == gtio)
    {
        /* GTCCRA buffer register. */
        GPT_CH(channel).GTCCRC = value;
    }
    else
    {
        /* GTCCRB buffer register. */
        GPT_CH(channel).GTCCRE = value;
    }
}

__STATIC_INLINE volatile void * HW_GPT_DutyCycleAddrGet(uint8_t const channel, gpt_gtioc_t gtio)
{
    volatile uint32_t * p_reg;
    if (GPT_GTIOCA == gtio)
    {
        /* GTCCRA buffer register. */
        p_reg = &GPT_CH(channel).GTCCRC;
    }
    else
    {
        /* GTCCRB buffer register. */
        p_reg = &GPT_CH(channel).GTCCRE;
    }
    return p_reg;
}

/*******************************************************************************************************************//**
 * Set how to change output at compare match.
 * @param   p_reg        Pointer to input/output control register
 * @param   output       Level to output at compare match
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_GTIOCCompareMatchOutputSet (gtior_t * const p_reg, gpt_output_t const output)
{
    p_reg->GTIOR_b.GTIOCM = output;
}

/*******************************************************************************************************************//**
 * Set initial output value
 * @param   p_reg        Pointer to input/output control register
 * @param   output       Level to output at compare match
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_GTIOCInitialOutputSet (gtior_t * const p_reg, gpt_pin_level_t const output)
{
    p_reg->GTIOR_b.GTIOCI = output;
}

/*******************************************************************************************************************//**
 * Select Additional Stop Source
 * @param   channel           The channel corresponds to the hardware channel number.
 * @param   source       Signal which triggers the action.
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_StopSourceSelectAdd (uint8_t const channel, gpt_trigger_t const source)
{
    GPT_CH(channel).GTPSR |= (uint32_t) source;
}
/*******************************************************************************************************************//**
 * Select Start Source
 * @param   channel           The channel corresponds to the hardware channel number.
 * @param   source       Signal which triggers the action.
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_StartSourceSelect(uint8_t const channel, gpt_trigger_t const source)
{
    GPT_CH(channel).GTSSR = source;
}

/*******************************************************************************************************************//**
 * Select Stop Source
 * @param   channel           The channel corresponds to the hardware channel number.
 * @param   source       Signal which triggers the action.
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_StopSourceSelect(uint8_t const channel, gpt_trigger_t const source)
{
    GPT_CH(channel).GTPSR = source;
}

/*******************************************************************************************************************//**
 * Select Clear Source
 * @param   channel           The channel corresponds to the hardware channel number.
 * @param   source       Signal which triggers the action.
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_ClearSourceSelect(uint8_t const channel, gpt_trigger_t const source)
{
    GPT_CH(channel).GTCSR = source;
}

/*******************************************************************************************************************//**
 * Select Capture-to-RegA Source
 * @param   channel           The channel corresponds to the hardware channel number.
 * @param   source       Signal which triggers the action.
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_CaptureASourceSelect(uint8_t const channel, gpt_trigger_t const source)
{
    GPT_CH(channel).GTICASR = source;
}

/*******************************************************************************************************************//**
 * Select Capture-to-RegB Source
 * @param   channel           The channel corresponds to the hardware channel number.
 * @param   source       Signal which triggers the action.
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_CaptureBSourceSelect(uint8_t const channel, gpt_trigger_t const source)
{
    GPT_CH(channel).GTICBSR = source;
}

/*******************************************************************************************************************//**
 * Return captured value from register GTCCRA
 * @param   channel           The channel corresponds to the hardware channel number.
 **********************************************************************************************************************/
__STATIC_INLINE uint32_t HW_GPT_RegisterAGet(uint8_t const channel)
{
    return GPT_CH(channel).GTCCRA;
}

/*******************************************************************************************************************//**
 * Initialize channel specific registers to default values.
 * @param   channel           The channel corresponds to the hardware channel number.
 **********************************************************************************************************************/
__STATIC_INLINE void HW_GPT_RegisterInit(uint8_t const channel)
{
    /* Skip these since they affect all channels, and are initialized in other registers: GTSTR, GTSTP, GTCLR. */
    GPT_CH(channel).GTWP = 0xA500;
    GPT_CH(channel).GTSSR = 0;
    GPT_CH(channel).GTPSR = 0;
    GPT_CH(channel).GTPSR = 0;
    GPT_CH(channel).GTUPSR = 0;
    GPT_CH(channel).GTDNSR = 0;
    GPT_CH(channel).GTICASR = 0;
    GPT_CH(channel).GTICBSR = 0;
    GPT_CH(channel).GTCR = 0;
    GPT_CH(channel).GTUDDTYC = 1;
    GPT_CH(channel).GTIOR = 0;
    GPT_CH(channel).GTINTAD = 0;
    GPT_CH(channel).GTST = 0;
    GPT_CH(channel).GTBER = 0;
    GPT_CH(channel).GTCNT = 0;
    GPT_CH(channel).GTCCRA = 0xFFFFFFFF;
    GPT_CH(channel).GTCCRB = 0xFFFFFFFF;
    GPT_CH(channel).GTCCRC = 0xFFFFFFFF;
    GPT_CH(channel).GTCCRE = 0xFFFFFFFF;
    GPT_CH(channel).GTPR = 0xFFFFFFFF;
    GPT_CH(channel).GTPBR = 0xFFFFFFFF;
    GPT_CH(channel).GTDTCR = 0;
    GPT_CH(channel).GTDVU = 0xFFFFFFFF;
#if BSP_MCU_GROUP_S7G2
    if (channel <= 7)
    {
        /* These registers available on GPTA only. */
        GPT_CH(channel).GTITC = 0;
        GPT_CH(channel).GTPDBR = 0xFFFFFFFF;
        GPT_CH(channel).GTADTRA = 0xFFFFFFFF;
        GPT_CH(channel).GTADTRB = 0xFFFFFFFF;
        GPT_CH(channel).GTADTBRA = 0xFFFFFFFF;
        GPT_CH(channel).GTADTBRB = 0xFFFFFFFF;
        GPT_CH(channel).GTADTDBRA = 0xFFFFFFFF;
        GPT_CH(channel).GTADTDBRB = 0xFFFFFFFF;
        GPT_CH(channel).GTDVD = 0xFFFFFFFF;
        GPT_CH(channel).GTDBU = 0xFFFFFFFF;
        GPT_CH(channel).GTDBD = 0xFFFFFFFF;
        /* GTSOS skipped - read only */
        GPT_CH(channel).GTSOTR = 0;
    }
#endif
}

#endif /* HW_GPT_COMMON_H */
